﻿#pragma once

#pragma region キー動作

#define	KEY_NOCHANGE	0xFFFFFFFF		// デフォルト

// キー動作(方向)(Tab,Enter)
#define	KEY_DIR_LEFT	0x00000000		// 左移動する
#define	KEY_DIR_RIGHT	0x00000001		// 右移動する
#define	KEY_DIR_DOWN	0x00000002		// 下移動する
#define	KEY_DIR_UP		0x00000003		// 上移動する
#define	KEY_DIR_MASK	0x00000003		// 移動マスク

// キー動作(次行or次列移動有無)(Tab,Enter,上,下,左,右)
#define	KEY_NEXT_TRUE	0x00000010		// 次行(次列)移動する
#define	KEY_NEXT_FALSE	0x00000000		// 次行(次列)移動しない
#define	KEY_NEXT_MASK	0x00000010		// 次行(次列)マスク

// キー動作(ローテーション有無)(Tab,Enter,上,下,左,右)
#define	KEY_ROT_TRUE	0x00000100		// ローテーションする
#define	KEY_ROT_FALSE	0x00000000		// ローテーションしない
#define	KEY_ROT_FOUT	0x00000200		// 他コントロール移動する(Tabキーのみ)
#define	KEY_ROT_MASK	0x00000300		// ローテーションマスク

#pragma endregion

#pragma region 通知

// 通知メッセージ
#define	LVNEX_CHANGEROW		NM_FIRST + 1u
#define	LVNEX_DELETEKEY		NM_FIRST + 2u
#define	LVNEX_COPYKEY		NM_FIRST + 3u
#define	LVNEX_PASTEKEY		NM_FIRST + 4u
#define	LVNEX_ACTIVECELL	NM_FIRST + 5u
#define	LVNEX_RCLICK		NM_FIRST + 6u

// uAction
#define	NTACTION_EDIT		1u
#define	NTACTION_ADD		2u
#define	NTACTION_DEL		3u
#define	NTACTION_COPY		4u
#define	NTACTION_PASTE		5u
#define	NTACTION_ACTIVECELL	6u
#define	NTACTION_CELL		7u
#define	NTACTION_COLHEADER	8u
#define	NTACTION_ROWHEADER	9u

#pragma endregion

#pragma region セル状態

#define	CELLST_NONE			0x0000
#define	CELLST_SELECTED		0x0001
#define	CELLST_FOCUSED		0x0002
#define	CELLST_ACTIVE		(CELLST_SELECTED | CELLST_FOCUSED)

#pragma endregion


namespace Koei
{

	// m_Align
	enum class HDEX_ALIGN {
		LISTCTRLSAME,
		LEFTALWAYS,
		CENTERALWAYS,
		RIGHTALWAYS,
	};

	// 行ヘッダー表示内容モード
	enum class CONTENTSMODE {
		AUTONUMBER,	// 自動連番(デフォルト)
		CUSTOM,		// 任意
	};

	// 選択状態
	enum class SELECTSTATUS {
		CELL,	// セル選択状態(デフォルト)
		LINE,	// 行選択状態
	};

	// 編集モード
	enum class EDITMODE {
		EDIT,	// 変更入力モード(デフォルト)
		ADD,	// 新規入力モード
	};

	// 行ステータス
	enum class LINESTATUS {
		NORMAL,
		EDITING,
		SELECTING,
		NEW,
	};

	// 編集開始時の入力カーソル状態
	enum class EDITCURSOR {
		LEFT,	// 左端(デフォルト)
		RIGHT,	// 右端
		SELECT,	// 選択
	};

	// 色情報用キー
	enum class COLOR_KEY {
		BACK,						// セル背景色
		BACK_READONLY,				// セル背景色(読み取り専用)
		BACK_SELECT,				// セル背景色(選択時)
		BACK_SELECT_LOST,			// セル背景色(選択時で非アクティブ)
		BORDER,						// 罫線色
		TEXT,						// セル文字色
		TEXT_READONLY,				// セル文字色(読み取り専用)
		TEXT_SELECT,				// セル文字色(選択時)
		TEXT_SELECT_LOST,			// セル文字色(選択時で非アクティブ)
		EDIT_BORDER,				// 編集コントロール境界線色
	};

#pragma region 通知

	typedef struct {
		NMHDR hdr;
		int iRow;
		int iCol;
		UINT uAction;
		size_t TextLength;
		TCHAR Text[1];
	} LVNEXNOTIFY, * LPLVNEXNOTIFY;

#pragma endregion

#pragma region ヒットテスト

	enum class HITTESTEXITEM {
		CELL,
		COLHEADER,
		ROWHEADER,
	};

	typedef struct {
		int iRow;
		int iCol;
		HITTESTEXITEM item;

	} LVNEXHITTEST, * LPLVNEXHITTEST;

#pragma endregion


#pragma region コールバック

	class CListCtrlEx;
	typedef BOOL(CALLBACK* PFBEFOREEDIT)(CListCtrlEx* pList, int iRow, int iCol, const CString& csSrc, CString& csDst, LPVOID pParam);
	typedef BOOL(CALLBACK* PFAFTEREDIT)(CListCtrlEx* pList, int iRow, int iCol, const CString& csSrc, CString& csDst, LPVOID pParam);
	typedef BOOL(CALLBACK* PFKEYENDEDIT)(CListCtrlEx* pList, int iRow, int iCol, int iKeyCode, BOOL bShiftKey, LPVOID pParam);
	typedef BOOL(CALLBACK* PFCLICKCELLENDEDIT)(CListCtrlEx* pList, int iRow, int iCol, int iClickRow, int iClickCol, BOOL bLeft, LPVOID pParam);
	typedef BOOL(CALLBACK* PFPRESSCHAR)(CListCtrlEx* pList, int iRow, int iCol, const WCHAR wSrc, WCHAR& wDst, LPVOID pParam);
	typedef BOOL(CALLBACK* PFAFTERPRESSCHAR)(CListCtrlEx* pList, int iRow, int iCol, const CString& csCheckText, LPVOID pParam);

#pragma endregion

#pragma region 動的にライブラリを読み込む

	typedef CListCtrlEx* (*PFINSTANCE)(const CString& csLincenseKey);
	extern "C" AFX_EXT_API CListCtrlEx * Instance(const CString & csLincenseKey);

#pragma endregion


	class AFX_EXT_CLASS CListCtrlEx : public CListCtrl
	{
	public:

#pragma region 作成

		/// <summary>
		/// インスタンス作成
		/// </summary>
		/// <param name="csLincenseKey"></param>
		/// <returns></returns>
		static CListCtrlEx* Instance(const CString& csLincenseKey);

		/// <summary>
		/// コントロール作成
		/// </summary>
		/// <param name="dwStyle">スタイル</param>
		/// <param name="rect">座標</param>
		/// <param name="pParentWnd">親ウィンドウ</param>
		/// <param name="nID">コントロールID</param>
		/// <returns></returns>
		virtual BOOL Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID) = 0;

#pragma endregion

#pragma region 列

		/// <summary>
		/// 追加(列)
		/// </summary>
		/// <param name="nCol">新しい列のインデックス</param>
		/// <param name="lpszColumnHeading">列名</param>
		/// <param name="bReadOnly">読取り専用</param>
		/// <param name="nFormat">表示位置</param>
		/// <param name="nWidth">列幅</param>
		/// <param name="nSubItem">列に関連付けられているサブ項目のインデックス</param>
		/// <returns>追加された列のインデックス</returns>
		virtual int InsertColumn(int nCol, LPCTSTR lpszColumnHeading, BOOL bReadOnly = FALSE, int nFormat = LVCFMT_LEFT, int nWidth = -1, int nSubItem = -1) = 0;

		/// <summary>
		/// タイプ設定(エディットコントロール)(列)
		/// </summary>
		/// <param name="nCol">列のインデックス</param>
		virtual void SetColumnEditType(int nCol) = 0;

		/// <summary>
		/// タイプ設定(チェックボックス)(列)
		/// </summary>
		/// <param name="nCol">列のインデックス</param>
		/// <param name="bDefault">新規行のチェック有無</param>
		virtual void SetColumnCheckType(int nCol, BOOL bDefaultCheck = FALSE) = 0;

		/// <summary>
		/// タイプ設定(コンボボックス)(列)
		/// </summary>
		/// <param name="nCol">列のインデックス</param>
		/// <param name="comboList">コンボボックスリスト</param>
		/// <param name="comboList">新規行の選択値</param>
		/// <param name="nMaxDropDownCount">ドロップダウン部分に表示される項目の最大数</param>
		virtual void SetColumnComboType(int nCol, CStringList* comboList, const CString& csDefaultValue, int nMaxDropDownCount = 10) = 0;

		/// <summary>
		/// 削除(列)
		/// </summary>
		/// <param name="nCol">列のインデックス</param>
		/// <returns>削除成否</returns>
		virtual BOOL DeleteColumn(int nCol) = 0;

		/// <summary>
		/// 色設定(列)
		/// </summary>
		/// <param name="iCol">列のインデックス -1:全列</param>
		/// <param name="colText">文字色</param>
		/// <param name="colBack">背景色</param>
		virtual void SetColColor(int iCol, COLORREF colText, COLORREF colBack) = 0;

		/// <summary>
		/// 色取得(列)
		/// </summary>
		/// <param name="iCol">列のインデックス</param>
		/// <param name="pcolText">文字色</param>
		/// <param name="pcolBack">背景色</param>
		/// <returns>TRUE:列の色設定あり FALSE:セルの色設定なし</returns>
		virtual BOOL GetColColor(int iCol, COLORREF* pcolText, COLORREF* pcolBack) = 0;

		/// <summary>
		/// 色クリア(列)
		/// </summary>
		/// <param name="iCol">列のインデックス -1:全列</param>
		virtual void ClearColColor(int iCol) = 0;

		/// <summary>
		/// 読み取り専用設定(列)
		/// </summary>
		/// <param name="iCol">列のインデックス -1:全列</param>
		/// <param name="bReadOnly">TRUE:読み取り専用にする FALSE:編集可能にする</param>
		virtual void SetColReadOnly(int iCol, BOOL bReadOnly) = 0;

		/// <summary>
		/// 読み取り専用設定取得
		/// </summary>
		/// <param name="iCol">列のインデックス</param>
		/// <returns>TRUE:読み取り専用設定している FALSE:読み取り専用設定していない</returns>
		virtual BOOL IsColReadOnly(int iCol) = 0;

#pragma endregion

#pragma region 列ヘッダー

		/// <summary>
		/// 高さ設定(列ヘッダー)
		/// </summary>
		/// <param name="iLineCount">行数</param>
		virtual void SetHeaderLineCount(int iLineCount) = 0;

		/// <summary>
		/// 高さ設定(列ヘッダー)
		/// </summary>
		/// <param name="iHeight">高さ</param>
		virtual void SetHeaderHeight(int iHeight) = 0;

		/// <summary>
		/// 高さ取得(列ヘッダー)
		/// </summary>
		/// <param name="piHeight">高さ</param>
		/// <return>行数 SetHeaderHeightを使用して高さを設定している場合は-1</return>
		virtual int GetHeaderHeight(int* piHeight) = 0;

		/// <summary>
		/// テキスト表示位置設定(列ヘッダー)
		/// </summary>
		/// <param name="Align">テキスト表示位置</param>
		virtual void SetHeaderAlign(HDEX_ALIGN Align) = 0;

		/// <summary>
		/// 幅変更可否設定(列ヘッダー)
		/// </summary>
		/// <param name="bEnable">変更可否</param>
		virtual void SetColumnEnableSizing(BOOL bEnable = FALSE) = 0;

		/// <summary>
		/// ドラッグドロップによる列並べ替え可否設定(列ヘッダー)
		/// </summary>
		/// <param name="bDragDrop">可否</param>
		virtual void SetHeaderDragDrop(BOOL bDragDrop = FALSE) = 0;

		/// <summary>
		/// フォント(ヘッダー)
		/// </summary>
		/// <param name="lpszFaceName">フォント名</param>
		/// <param name="iFontSize">フォントサイズ</param>
		/// <param name="bBold">太字</param>
		/// <param name="bItalic">斜体</param>
		virtual void SetHeaderFont(LPCTSTR lpszFaceName, int iFontSize, BOOL bBold = FALSE, BOOL bItalic = FALSE) = 0;

		/// <summary>
		/// フォント(ヘッダー)
		/// </summary>
		/// <param name="pLogFont">ログフォント</param>
		virtual void SetHeaderFont(const LOGFONT* pLogFont) = 0;

		/// <summary>
		/// フォント取得(ヘッダー)
		/// </summary>
		/// <param name="pLogFont">ログフォント</param>
		virtual void GetHeaderLogFont(LOGFONT* pLogFont) = 0;

#pragma endregion

#pragma region 行

		/// <summary>
		/// 追加(行)
		/// </summary>
		/// <param name="iRow">挿入する行のインデックス</param>
		/// <param name="lpszItem">内容</param>
		/// <param name="bReadOnly">読み取り専用</param>
		/// <returns>新しい行のインデックス</returns>
		virtual int InsertItem(int iRow, LPCTSTR lpszItem, BOOL bReadOnly = FALSE) = 0;

		/// <summary>
		/// 追加(行)
		/// </summary>
		/// <param name="nMask">行の属性マスク</param>
		/// <param name="iRow">挿入する行のインデックス</param>
		/// <param name="lpszItem">内容</param>
		/// <param name="nState">行の状態</param>
		/// <param name="nStateMask">行の状態マスク</param>
		/// <param name="lParam">行に関連付けられている32ビットアプリケーション固有の値</param>
		/// <param name="bReadOnly">読み取り専用</param>
		/// <returns>新しい行のインデックス</returns>
		virtual int InsertItem(UINT nMask, int iRow, LPCTSTR lpszItem, UINT nState, UINT nStateMask, LPARAM lParam, BOOL bReadOnly = FALSE) = 0;

		/// <summary>
		/// 削除(行)
		/// </summary>
		/// <param name="iRow">削除する行のインデックス</param>
		/// <returns>成否</returns>
		virtual BOOL DeleteItem(int iRow) = 0;

		/// <summary>
		/// 全削除(行)
		/// </summary>
		/// <returns>成否</returns>
		virtual BOOL DeleteAllItems() = 0;

		/// <summary>
		/// 行選択(行)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="bSelect">選択/解除</param>
		virtual void SelectLine(int iRow, BOOL bSelect = TRUE) = 0;

		/// <summary>
		/// クリア(行)
		/// </summary>
		/// <param name="iRow">行のインデックス -1:選択行</param>
		/// <param name="bForce">TRUE:読み取り専用でもクリアする FALSE:読み取り専用セルはクリアしない</param>
		virtual void ClearRow(int iRow = -1, BOOL bForce = FALSE) = 0;

		/// <summary>
		/// 全クリア(行)
		/// </summary>
		/// <param name="bForce">TRUE:読み取り専用でもクリアする FALSE:読み取り専用セルはクリアしない</param>
		virtual void ClearAll(BOOL bForce = FALSE) = 0;

		/// <summary>
		/// 読み取り専用設定(行)
		/// </summary>
		/// <param name="iRow">行のインデックス -1:全行</param>
		/// <param name="bReadOnly">TRUE:設定 FALSE:解除</param>
		virtual void SetRowReadOnly(int iRow, BOOL bReadOnly) = 0;

		/// <summary>
		/// 読み取り専用設定取得(行)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <returns>TRUE:読み取り専用設定している FALSE:読み取り専用設定していない</returns>
		virtual BOOL IsRowReadOnly(int iRow) = 0;

		/// <summary>
		/// 高さ設定(行)
		/// </summary>
		/// <param name="iHeight">高さ -1:自動</param>
		virtual void SetRowHeight(int iHeight) = 0;

		/// <summary>
		/// 高さ取得(行)
		/// </summary>
		/// <returns>高さ</returns>
		virtual int GetRowHeight() = 0;

		/// <summary>
		/// 状態取得(行)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <returns>状態</returns>
		virtual LINESTATUS GetLineStatus(int iRow) = 0;

		/// <summary>
		/// ソート関数
		/// </summary>
		/// <param name="pfnCompare"></param>
		/// <param name="dwData"></param>
		/// <returns></returns>
		virtual BOOL SortItems(PFNLVCOMPARE pfnCompare, DWORD_PTR dwData) = 0;

		/// <summary>
		/// ソート関数
		/// </summary>
		/// <param name="pfnCompare"></param>
		/// <param name="dwData"></param>
		/// <returns></returns>
		virtual BOOL SortItemsEx(PFNLVCOMPARE pfnCompare, DWORD_PTR dwData) = 0;

#pragma endregion

#pragma region 行ヘッダー

		/// <summary>
		/// 幅設定(行ヘッダー)
		/// </summary>
		/// <param name="iWidth">幅 -1:行ヘッダー無し</param>
		virtual void SetRowHeaderWidth(int iWidth) = 0;

		/// <summary>
		/// 幅取得(行ヘッダー)
		/// </summary>
		/// <returns>幅 -1:行ヘッダー無し</returns>
		virtual int GetRowHeaderWidth() = 0;

		/// <summary>
		/// テキスト表示位置設定(行ヘッダー)
		/// </summary>
		/// <param name="uAlign">テキスト表示位置</param>
		virtual void SetRowHeaderAlign(UINT uAlign) = 0;

		/// <summary>
		/// テキストモード設定(行ヘッダー)
		/// </summary>
		/// <param name="contentsMode">テキスト表示内容モード</param>
		virtual void SetRowHeaderContentsMode(CONTENTSMODE contentsMode) = 0;

		/// <summary>
		/// テキスト設定(行ヘッダー)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="lpszContents">内容</param>
		/// <returns>成否</returns>
		virtual BOOL SetRowHeaderContents(int iRow, LPCTSTR lpszContents) = 0;

		/// <summary>
		/// 再描画(行ヘッダー)
		/// </summary>
		virtual void RedrawRowHeader() = 0;

#pragma endregion

#pragma region セル

		/// <summary>
		/// アクティブセル設定(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス</param>
		/// <param name="bVisible">TRUE:スクロールしてセルを表示するようにする</param>
		/// <param name="selectStatus">選択状態</param>
		/// <returns>成否</returns>
		virtual BOOL SetActiveCell(int iRow, int iCol, BOOL bVisible = TRUE, SELECTSTATUS selectStatus = SELECTSTATUS::CELL) = 0;

		/// アクティブセル取得(セル)
		/// </summary>
		/// <param name="piRow">行のインデックス</param>
		/// <param name="piCol">列のインデックス</param>
		/// <param name="bSelected">アクティブセルの判定方法</param>
		/// <returns>TRUE:アクティブセルあり FALSE:アクティブセルなし</returns>
		virtual BOOL GetActiveCell(int* piRow, int* piCol, BOOL bSelected = FALSE) = 0;

		/// <summary>
		/// アクティブセルクリア(セル)
		/// </summary>
		virtual void ClearActiveCell() = 0;

		/// <summary>
		/// セル状態取得(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス</param>
		/// <returns>状態</returns>
		virtual UINT GetCellState(int iRow, int iCol) = 0;

		/// <summary>
		/// 編集開始(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス</param>
		/// <returns>成否</returns>
		virtual BOOL StartEdit(int iRow = -1, int iCol = -1) = 0;

		/// <summary>
		/// 編集確定終了(セル)
		/// </summary>
		virtual void EndEdit() = 0;

		/// <summary>
		/// 編集キャンセル終了(セル)
		/// </summary>
		virtual void CancelEdit() = 0;

		/// <summary>
		/// 編集中確認
		/// </summary>
		/// <returns>TRUE:編集中 FALSE:編集中ではない</returns>
		virtual BOOL IsEditing() = 0;

		/// <summary>
		/// クリア(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス -1:選択行</param>
		/// <param name="iCol">列のインデックス -1:選択列</param>
		/// <param name="bForce">TRUE:読み取り専用でもクリアする FALSE:読み取り専用セルはクリアしない</param>
		virtual void ClearCell(int iRow = -1, int iCol = -1, BOOL bForce = FALSE) = 0;

		/// <summary>
		/// リセット(セル)
		/// </summary>
		virtual void Reset() = 0;

		/// <summary>
		/// 読み取り専用設定(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス</param>
		/// <param name="bReadOnly">TRUE:設定 FALSE:解除</param>
		virtual void SetCellReadOnly(int iRow, int iCol, BOOL bReadOnly) = 0;

		/// <summary>
		/// 読み取り専用設定取得(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス</param>
		/// <returns>TRUE:読み取り専用設定している FALSE:読み取り専用設定していない</returns>
		virtual BOOL IsCellReadOnly(int iRow, int iCol) = 0;

		/// <summary>
		/// 色設定(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス -1:全列</param>
		/// <param name="colText">文字色</param>
		/// <param name="colBack">背景色</param>
		virtual void SetCellColor(int iRow, int iCol, COLORREF colText, COLORREF colBack) = 0;

		/// <summary>
		/// 色取得(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス</param>
		/// <param name="pcolText">文字色</param>
		/// <param name="pcolBack">背景色</param>
		/// <returns>TRUE:列の色設定あり FALSE:セルの色設定なし</returns>
		virtual BOOL GetCellColor(int iRow, int iCol, COLORREF* pcolText, COLORREF* pcolBack) = 0;

		/// <summary>
		/// 表示色取得(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス</param>
		/// <param name="pcolText">文字色</param>
		/// <param name="pcolBack">背景色</param>
		/// <returns>TRUE:正常取得 FALSE:取得失敗</returns>
		virtual BOOL GetCellDisplayColor(int iRow, int iCol, COLORREF* pcolText, COLORREF* pcolBack) = 0;

		/// <summary>
		/// 色クリア(セル)
		/// </summary>
		/// <param name="iRow">行のインデックス</param>
		/// <param name="iCol">列のインデックス -1:全列</param>
		virtual void ClearCellColor(int iRow, int iCol) = 0;

		/// <summary>
		/// フォント設定(セル)
		/// </summary>
		/// <param name="lpszFaceName">フォント名</param>
		/// <param name="nPointSize">フォントサイズ</param>
		/// <param name="bBold">太字</param>
		/// <param name="bItalic">斜体</param>
		virtual void SetCellFont(LPCTSTR lpszFaceName, int nPointSize, BOOL bBold = FALSE, BOOL bItalic = FALSE) = 0;

		/// <summary>
		/// フォント設定(セル)
		/// </summary>
		/// <param name="pLogFont">ログフォント</param>
		virtual void SetCellFont(const LOGFONT* pLogFont) = 0;

		/// <summary>
		/// フォント取得(セル)
		/// </summary>
		/// <param name="pLogFont">ログフォント</param>
		virtual void GetCellFont(LOGFONT* pLogFont) = 0;

#pragma endregion

#pragma region 各種情報の設定/取得

		/// <summary>
		/// キー動作設定
		/// </summary>
		/// <param name="uTabKey">タブキーの動作</param>
		/// <param name="uEnterKey">Enterキー動作</param>
		/// <param name="uUpKey">上キー動作</param>
		/// <param name="uDownKey">下キー動作</param>
		/// <param name="uLeftKey">左キー動作</param>
		/// <param name="uRightKey">右キー動作</param>
		virtual void SetKeyMotion(UINT uTabKey, UINT uEnterKey, UINT uUpKey, UINT uDownKey, UINT uLeftKey, UINT uRightKey) = 0;

		/// <summary>
		/// キー動作取得
		/// </summary>
		/// <param name="puTabKey">タブキーの動作</param>
		/// <param name="puEnterKey">Enterキー動作</param>
		/// <param name="puUpKey">上キー動作</param>
		/// <param name="puDownKey">下キー動作</param>
		/// <param name="puLeftKey">左キー動作</param>
		/// <param name="puRightKey">右キー動作</param>
		virtual void GetKeyMotion(UINT* puTabKey, UINT* puEnterKey, UINT* puUpKey, UINT* puDownKey, UINT* puLeftKey, UINT* puRightKey) = 0;

		/// <summary>
		/// 編集モード設定
		/// </summary>
		/// <param name="eEditMode">編集モード</param>
		virtual void SetEditMode(EDITMODE eEditMode) = 0;

		/// <summary>
		/// 編集モード取得
		/// </summary>
		/// <returns>編集モード</returns>
		virtual EDITMODE GetEditMode() = 0;

		/// <summary>
		/// 選択状態取得
		/// </summary>
		/// <returns>選択状態</returns>
		virtual SELECTSTATUS GetSelectStatus() = 0;

		/// <summary>
		/// 行選択可否設定
		/// </summary>
		/// <param name="bEnable">行選択可否</param>
		virtual void EnableLineSelect(BOOL bEnable) = 0;

		/// <summary>
		/// 変更状態取得
		/// </summary>
		/// <returns>TRUE:変更あり FALSE:変更なし</returns>
		virtual BOOL IsModify() = 0;

		/// <summary>
		/// 変更状態設定
		/// </summary>
		/// <param name="bModify">TRUE:変更あり FALSE:変更なし</param>
		virtual void SetModify(BOOL bModify = TRUE) = 0;

		/// <summary>
		/// 編集開始時カーソル状態設定
		/// </summary>
		/// <param name="eEditCursor">カーソル状態</param>
		virtual void SetEditCursor(EDITCURSOR eEditCursor) = 0;

		/// <summary>
		/// 編集開始時カーソル状態取得
		/// </summary>
		/// <returns>カーソル状態</returns>
		virtual EDITCURSOR GetEditCursor() = 0;

		/// <summary>
		/// 基本色取得
		/// </summary>
		/// <param name="ColorKey">取得する色の種類</param>
		/// <returns>色</returns>
		virtual COLORREF GetBaseColor(COLOR_KEY ColorKey) = 0;

		/// <summary>
		/// 基本色設定
		/// </summary>
		/// <param name="ColorKey">設定する色の種類</param>
		/// <param name="color">色</param>
		virtual void SetBaseColor(COLOR_KEY ColorKey, COLORREF color) = 0;

		/// <summary>
		/// コントロール矩形取得
		/// </summary>
		/// <param name="lpRect">スクリーン座標</param>
		virtual void GetWindowRectWithRowHeader(LPRECT lpRect) = 0;

		/// <summary>
		/// コントロール移動を
		/// </summary>
		/// <param name="lpRect">移動先のクライアント座標</param>
		/// <param name="bRepaint">TRUE:再描画する FALSE:再描画しない</param>
		virtual void MoveWindowWithRowHeader(LPRECT lpRect, BOOL bRepaint = TRUE) = 0;

		/// <summary>
		/// コントロールサイズ変更
		/// </summary>
		/// <param name="iWidh">幅</param>
		/// <param name="iHeight">高さ</param>
		virtual void Fit(int iWidh, int iHeight) = 0;

		/// <summary>
		/// コントロールサイズ変更
		/// </summary>
		/// <param name="iRightMargin">右マージン</param>
		/// <param name="iBottomMargin">下マージン</param>
		virtual void FitRBMargin(int iRightMargin, int iBottomMargin) = 0;

		/// <summary>
		/// ヒットテスト
		/// </summary>
		/// <param name="pt">テストするスクリーン座標</param>
		/// <param name="info">結果</param>
		/// <returns>TRUE:結果あり FALSE:結果なし</returns>
		virtual BOOL HitTestEx(CPoint pt, LPLVNEXHITTEST info) = 0;

		/// <summary>
		/// キャンセルボタン設定
		/// </summary>
		/// <param name="pWnd">編集キャンセルするウィンドウ</param>
		/// <param name="bAdd">TRUE:追加する FALSE:削除する</param>
		virtual void SetCancelWnd(CWnd* pWnd, BOOL bAdd = TRUE) = 0;

		/// <summary>
		/// キャンセルボタン設定クリア
		/// </summary>
		virtual void ClearCancelWnd() = 0;

		/// <summary>
		/// 終了設定
		/// </summary>
		virtual void Finish() = 0;

		/// <summary>
		/// バージョン番号取得
		/// </summary>
		/// <returns></returns>
		static CString GetVersion();

#pragma endregion

#pragma region コールバック関数

		/// <summary>
		/// 編集開始時
		/// </summary>
		/// <param name="pfBeforeEdit">編集開始時に呼ぶコールバック関数</param>
		/// <param name="pParam">コールバック関数に渡されるポインタ</param>
		virtual void SetBeforeEditFunc(PFBEFOREEDIT pfBeforeEdit, LPVOID pParam = NULL) = 0;

		/// <summary>
		/// 編集終了時
		/// </summary>
		/// <param name="pfAfterEdit">編集終了時に呼ぶコールバック関数</param>
		/// <param name="pParam">コールバック関数に渡されるポインタ</param>
		virtual void SetAfterEditFunc(PFAFTEREDIT pfAfterEdit, LPVOID pParam = NULL) = 0;

		/// <summary>
		/// Enter,Tabキーによる編集終了後
		/// </summary>
		/// <param name="pfKeyEndEdit">Enter,Tabキーによる編集終了後に呼ぶコールバック関数</param>
		/// <param name="pParam">コールバック関数に渡されるポインタ</param>
		virtual void SetKeyEndEditFunc(PFKEYENDEDIT pfKeyEndEdit, LPVOID pParam = NULL) = 0;

		/// <summary>
		/// セルクリックによる編集終了後
		/// </summary>
		/// <param name="pfClickCellEndEdit">セルクリックによる編集終了後に呼ぶコールバック関数</param>
		/// <param name="pParam">コールバック関数に渡されるポインタ</param>
		virtual void SetClickCellEndEditFunc(PFCLICKCELLENDEDIT pfClickCellEndEdit, LPVOID pParam = NULL) = 0;

		/// <summary>
		/// 文字入力時
		/// </summary>
		/// <param name="pfPressChar">文字入力時に呼ぶコールバック関数</param>
		/// <param name="pParam">コールバック関数に渡されるポインタ</param>
		virtual void SetPressCharFunc(PFPRESSCHAR pfPressChar, LPVOID pParam = NULL) = 0;

		/// <summary>
		/// 文字入力後
		/// </summary>
		/// <param name="pfAfterPressChar">文字入力後に呼ぶコールバック関数</param>
		/// <param name="pParam">コールバック関数に渡されるポインタ</param>
		virtual void SetAfterPressCharFunc(PFAFTERPRESSCHAR pfAfterPressChar, LPVOID pParam = NULL) = 0;

#pragma endregion

	};

}


